/***************************************************************************
                          xineconfig.cpp - config dialog for xine parameters
                             -------------------
    begin                : Son Mai 4 2003
    revision             : $Revision: 1.1 $
    last modified        : $Date: 2004/09/15 15:06:38 $ by $Author: juergenk $
    copyright            : (C) 2003-2004 by Jürgen Kofler
    email                : kaffeine@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kglobal.h>
#include <kiconloader.h>
#include <klineedit.h>
#include <kcombobox.h>
#include <kpushbutton.h>
#include <kdialogbase.h>
#include <klocale.h>
#include <kdebug.h>
#include <kseparator.h>

#include <qwidget.h>
#include <qlayout.h>
#include <qvbox.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qptrlist.h>
#include <qstringlist.h>
#include <qlabel.h>
#include <qscrollview.h>
#include <qsignalmapper.h>

#include "xineconfig.h"
#include "xineconfig.moc"

#define NON_EXPERT_OPTIONS "audio.speaker_arrangement;audio.driver;audio.mixer_software;video.driver;dxr3.device_number;dxr3.enc_add_bars;dxr3.enc_alt_play_mode;input.dvd_language;input.dvd_region;input.cdda_device;input.cdda_use_cddb;input.drive_slowdown;input.dvd_device;input.vcd_device;input.http_no_proxy;input.http_proxy_host;input.http_proxy_password;input.http_proxy_port;input.http_proxy_user;codec.real_codecs_path;codec.win32_path;post.goom_fps;post.goom_height;post.goom_width;misc.spu_subtitle_size;misc.spu_vertical_offset;misc.spu_src_encoding;misc.sub_timeout;misc.osd_messages;vcd.default_device"

XineConfigEntry::XineConfigEntry(QWidget* parent, QGridLayout* grid, int row, xine_cfg_entry_t* entry) :
  m_valueChanged(false), m_key(QString(entry->key)), m_numValue(entry->num_value), m_stringValue(entry->str_value)
{
  switch (entry->type)
  {
    case XINE_CONFIG_TYPE_UNKNOWN: break;
    case XINE_CONFIG_TYPE_STRING:
    {
      m_stringEdit = new KLineEdit(entry->str_value, parent);
      grid->addWidget(m_stringEdit, row ,0);
      connect(m_stringEdit, SIGNAL(textChanged(const QString&)), this, SLOT(slotStringChanged(const QString&)));
      break;
    }
    case XINE_CONFIG_TYPE_ENUM:
    {
      m_enumEdit = new KComboBox(parent);
      int i = 0;
      while (entry->enum_values[i])
      {
        m_enumEdit->insertItem(entry->enum_values[i]);
        i++;
      }
      m_enumEdit->setCurrentItem(entry->num_value);
      grid->addWidget(m_enumEdit, row, 0);
      connect(m_enumEdit, SIGNAL(activated(int)), this, SLOT(slotNumChanged(int)));
      break;
    }
    case XINE_CONFIG_TYPE_NUM:
    {
      m_numEdit = new QSpinBox(-999999, 999999, 1, parent);
      m_numEdit->setValue(entry->num_value);
      grid->addWidget(m_numEdit, row, 0);
      connect(m_numEdit, SIGNAL(valueChanged(int)), this, SLOT(slotNumChanged(int)));
      break;
    }
    case XINE_CONFIG_TYPE_RANGE:
    {
      m_numEdit = new QSpinBox(parent);
      m_numEdit->setValue(entry->num_value);
      m_numEdit->setRange(entry->range_min, entry->range_max);
      grid->addWidget(m_numEdit, row, 0);
      connect(m_numEdit, SIGNAL(valueChanged(int)), this, SLOT(slotNumChanged(int)));
      break;
    }       
    case XINE_CONFIG_TYPE_BOOL:
    {
      m_boolEdit = new QCheckBox(parent);
      m_boolEdit->setChecked(entry->num_value);
      grid->addWidget(m_boolEdit, row, 0);
      connect(m_boolEdit, SIGNAL(toggled(bool)), this, SLOT(slotBoolChanged(bool)));
      break;
    }        
  }

  QString m_keyName(entry->key);
  m_keyName.remove( 0, m_keyName.find(".") + 1 );

  QLabel* description = new QLabel(m_keyName + "\n" + QString::fromUtf8(entry->description), parent);
  description->setAlignment( QLabel::WordBreak | QLabel::AlignVCenter );
  grid->addWidget(description, row, 1);

  KSeparator* separator = new KSeparator(KSeparator::Horizontal, parent);
  grid->addMultiCellWidget(separator, row+1, row+1, 0, 1);
}

XineConfigEntry::~XineConfigEntry() {}

void XineConfigEntry::slotNumChanged(int val)
{
   m_numValue = val;
   m_valueChanged = true;
}

void XineConfigEntry::slotBoolChanged(bool val) 
{
   m_numValue = val;
   m_valueChanged = true;
}

void XineConfigEntry::slotStringChanged(const QString& val)
{
   m_stringValue = val;
   m_valueChanged = true;
}

bool XineConfigEntry::valueChanged() const
{
  return m_valueChanged;
}
  
const QString& XineConfigEntry::getKey() const
{
  return m_key;
}
  
int XineConfigEntry::getNumValue() const
{
  return m_numValue;
}
  
const QString& XineConfigEntry::getStringValue() const
{
  return m_stringValue;
}  


/************** XINE CONFIG CLASS **************************/

XineConfig::XineConfig(const xine_t* const xine) :
       KDialogBase(KDialogBase::IconList, i18n("xine Engine Parameters"), KDialogBase::Ok|KDialogBase::Apply|KDialogBase::Cancel, KDialogBase::Cancel)

{
  setInitialSize(QSize(650,500), true);

  m_xine = (xine_t*)xine;
  
  QStringList cats = getCategories();
  QVBox* xinePage = NULL;
  QString icon;
  KPushButton* expert = NULL;
  
  QSignalMapper* mapper = new QSignalMapper(this);
  connect(mapper, SIGNAL(mapped(const QString&)), this, SLOT(slotShowExpertOptions(const QString&)));
  
  for (QStringList::ConstIterator it = cats.begin(); it != cats.end(); it++)
  { 
   //  kdDebug() << "XineConfig: add page: " << *it << endl;
     if (*it == "audio") icon = "sound";
     else if (*it == "video") icon = "video";
     else if (*it == "vcd") icon = "cdrom_unmount";
     else if (*it == "input") icon = "connect_established";
     else icon = "edit";
     xinePage = addVBoxPage(*it, i18n("%1 Options").arg(*it), KGlobal::iconLoader()->loadIcon(icon, KIcon::Panel, KIcon::SizeMedium));
     xinePage->setMargin(5);
     createPage(*it, false, xinePage);
     expert = new KPushButton(i18n("Expert Options"), xinePage);
     expert->setSizePolicy(QSizePolicy (QSizePolicy::Fixed, QSizePolicy::Fixed));
     connect(expert, SIGNAL(clicked()), mapper, SLOT(map()));
     mapper->setMapping(expert, *it);
  }
 
  connect(this, SIGNAL(okClicked()), SLOT(slotOkPressed()));
  connect(this, SIGNAL(applyClicked()), SLOT(slotApplyPressed()));
}

XineConfig::~XineConfig()
{
  m_entries.setAutoDelete(true); 
  m_entries.clear();
  kdDebug() << "XineConfig: destructed" << endl;
}

void XineConfig::createPage(const QString& cat, bool expert, QWidget* parent)
{
  xine_cfg_entry_t* ent;
  
  QScrollView* sv = new QScrollView(parent);
  sv->setResizePolicy(QScrollView::AutoOneFit);
  parent = new QWidget(sv->viewport());
  sv->addChild(parent); 
   
  QGridLayout* grid = new QGridLayout(parent, 20 ,2);
  grid->setColStretch(1,8);
  grid->setSpacing(10);
  grid->setMargin(10);
  
  uint row = 0;
  QString entCat;

/*********** read in xine config entries ***********/    
  ent = new xine_cfg_entry_t; 
  xine_config_get_first_entry(m_xine, ent);

  do
  {
    entCat = QString(ent->key);
    entCat = entCat.left(entCat.find("."));
    if (entCat == cat)
    {
      if (((!expert) && (QString(NON_EXPERT_OPTIONS).contains(ent->key))) ||
         ((expert) && (!QString(NON_EXPERT_OPTIONS).contains(ent->key))))
      {
        m_entries.append(new XineConfigEntry(parent, grid, row, ent));
        delete ent;
        ent = new xine_cfg_entry_t;   
        row += 2;
      }   
    }  
  } while(xine_config_get_next_entry(m_xine, ent));

  delete ent;
}

const QStringList XineConfig::getCategories()
{
  QStringList cats;
  xine_cfg_entry_t* ent = new xine_cfg_entry_t; 
  if (!xine_config_get_first_entry(m_xine, ent)) 
    return cats;
  QString entCat;
  
  do
  {
    entCat = QString(ent->key);
    entCat = entCat.left(entCat.find("."));  
    if (cats.findIndex(entCat) == -1)
    {
     // kdDebug() << "XineConfig: new category: " << entCat << endl;
      cats.append(entCat);
    }  
    delete ent;
    ent = new xine_cfg_entry_t;  
    
  } while(xine_config_get_next_entry(m_xine, ent));

  delete ent;
  return cats;
}

void XineConfig::slotShowExpertOptions(const QString& category)
{
  kdDebug() << "XineConfig: Show expert options for " << category << endl;
  
  KDialogBase* dialog = new KDialogBase( 0, "expertoptions", true, i18n("%1 Expert Options").arg(category), KDialogBase::Ok|KDialogBase::Cancel );
  QVBox* page = dialog->makeVBoxMainWidget();
  dialog->setInitialSize(QSize(500,400));
  
  createPage(category, true, page);
  
  int button = dialog->exec();
  if (button == KDialogBase::Accepted)
  {
    slotApplyPressed();
  }
     
  delete dialog;
}

void XineConfig::slotOkPressed()
{
  slotApplyPressed();
  QDialog::close();
}

/*************** apply changed entries *****************/

void XineConfig::slotApplyPressed()
{
  xine_cfg_entry_t *entry;

  
  for(uint i = 0; i < m_entries.count(); i++) /* check all entries */
  {
    if (m_entries.at(i)->valueChanged())  /* changed? */
    {
      entry = new xine_cfg_entry_t;
      if (xine_config_lookup_entry(m_xine, m_entries.at(i)->getKey(), entry))
      {
        kdDebug() << "XineConfig: Apply: " << m_entries.at(i)->getKey() << "\n";
     
        entry->num_value = m_entries.at(i)->getNumValue();
        
        if (m_entries.at(i)->getStringValue()) 
          entry->str_value = (char*) (const char*)m_entries.at(i)->getStringValue().latin1(); 

        xine_config_update_entry(m_xine, entry);            
        delete entry;

        m_entries.at(i)->setValueUnchanged();   
      }    
    }
  }    
}


