/***************************************************************************
 *   Copyright (C) 2004 by Jürgen Kofler                                   *
 *   kaffeine@gmx.net                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
 /*
  * Last modified: $Date: 2004/11/20 17:23:38 $ by $Author: juergenk $
  */

#include "pref.h"
#include "pref.moc"

#include <kglobal.h>
#include <kiconloader.h>
#include <kpushbutton.h>
#include <klocale.h>

#include <qlayout.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qcombobox.h>

#include <qtextcodec.h>

KaffeinePreferences::KaffeinePreferences()
    : KDialogBase(IconList,i18n("Kaffeine Setup"),
                  Ok|Apply|Cancel, Ok)
{
    setInitialSize(QSize(400,400), false);
    QFrame *general = addPage(i18n("General"), i18n("General Options"),
           KGlobal::iconLoader()->loadIcon("configure", KIcon::Panel, KIcon::SizeMedium));
    QGridLayout* generalGrid = new QGridLayout(general, 6, 2);
    generalGrid->setSpacing(10);
    
    m_systemTray = new QCheckBox(i18n("Embed in system tray"), general);
    connect(m_systemTray, SIGNAL(toggled(bool)), this, SLOT(slotEmbedInTrayToggled(bool)));
    generalGrid->addMultiCellWidget(m_systemTray, 0, 0, 0, 1);
    
    m_osdTimeout = new QSpinBox(general);
    m_osdTimeout->setMinValue(0);
    m_osdTimeout->setMaxValue(60);
    m_osdTimeout->setSuffix(i18n(" sec"));
    m_osdTimeout->setSpecialValueText(i18n("off"));
    generalGrid->addWidget(m_osdTimeout, 1,0);
    QLabel* osdTimeoutLabel = new QLabel(i18n("Duration of title announcement in system tray"), general);
    generalGrid->addWidget(osdTimeoutLabel, 1,1);
    
    
    m_showGoTab = new QCheckBox(i18n("Create Go!-Tab"), general);
    generalGrid->addMultiCellWidget(m_showGoTab, 2, 2, 0, 1);

    m_useAlternateEncoding = new QCheckBox(i18n("Use alternate (non-Unicode) encoding for Meta tags"), general);
    connect(m_useAlternateEncoding, SIGNAL(toggled(bool)), this, SLOT(slotUseAlternateEncodingToggled(bool)));
    generalGrid->addMultiCellWidget(m_useAlternateEncoding, 3, 3, 0, 1);
    m_alternateEncoding = new QComboBox(general);

//*** addind Western, European & Middle-Eastern Codecs to the combo box ***//
    QTextCodec *codec;
    QString codecName;
    QStringList codecNames;
    for (int i = 0; (codec = QTextCodec::codecForIndex(i)); i++) {
       codecName = codec->name();
       if ((codecName.left(2) == "CP") || (codecName.left(4) == "ISO ") || (codecName.left(3) == "IBM") || (codecName.left(3) == "KOI"))
       {
          codecNames.append(codecName);
       }
    }
    codecNames.sort();
    m_alternateEncoding->insertStringList(codecNames);
    generalGrid->addWidget(m_alternateEncoding, 4, 0);

    QLabel* encodingLabel = new QLabel(i18n("Choose alternate encoding name for Meta tags\n(in order to convert to Unicode)"), general);
    generalGrid->addWidget(encodingLabel, 4, 1);

    m_timeout = new QSpinBox(general);
    m_timeout->setMinValue(0);
    m_timeout->setMaxValue(10);
    m_timeout->setSuffix(i18n(" min"));
    m_timeout->setSpecialValueText(i18n("off"));
    generalGrid->addWidget(m_timeout, 5,0);
    QLabel* timeoutLabel = new QLabel(i18n("Interval to fake keypresses\n(prevent screensaver from activating)"), general);
    generalGrid->addWidget(timeoutLabel, 5,1);
#ifndef HAVE_XTEST    
    m_timeout->setEnabled(false);    
    timeoutLabel->setEnabled(false);
#endif

    KPushButton* clearRecent = new KPushButton( i18n("Clear"), general);
    clearRecent->setSizePolicy( QSizePolicy (QSizePolicy::Minimum, QSizePolicy::Fixed));
    connect(clearRecent, SIGNAL(clicked()), this, SIGNAL(signalClearRecent()));
    generalGrid->addWidget(clearRecent, 6, 0);
    QLabel* clearLabel = new QLabel(i18n("Clear recent files list"), general);
    generalGrid->addWidget(clearLabel, 6, 1);
    
    connect(this, SIGNAL(okClicked()), SLOT(slotOkPressed()));
    connect(this, SIGNAL(applyClicked()), SLOT(slotApplyPressed()));
}

void KaffeinePreferences::setConfig(int timeout, bool tray, uint duration, bool showGoTab, bool useEncoding, const QString& encoding)
{
  m_timeout->setValue(timeout);
  m_osdTimeout->setValue(duration);
  m_osdTimeout->setEnabled(tray);
  m_systemTray->setChecked(tray);
  m_showGoTab->setChecked(showGoTab);
  m_useAlternateEncoding->setChecked(useEncoding);
  m_alternateEncoding->setEnabled(useEncoding);
  m_alternateEncoding->setCurrentText(encoding);
}

void KaffeinePreferences::slotApplyPressed()
{
  emit signalTimeoutInterval(m_timeout->value());
  emit signalEmbedSystemTray(m_systemTray->isChecked());
  emit signalUseAlternateEncoding(m_useAlternateEncoding->isChecked());
  emit signalAlternateEncoding(m_alternateEncoding->currentText());
  emit signalShowGoTab(m_showGoTab->isChecked());
  emit signalSetOSDTimeout(m_osdTimeout->value());
}

void KaffeinePreferences::slotOkPressed()
{
  slotApplyPressed();
  hide();
}

void KaffeinePreferences::slotUseAlternateEncodingToggled(bool on)
{
  m_alternateEncoding->setEnabled(on);
}  

void KaffeinePreferences::slotEmbedInTrayToggled(bool on)
{
  m_osdTimeout->setEnabled(on);
}


